% MESH AND TRIANGULATION
% Purpose: n-dim'l mappings for operability purposes
% Description: from input-output data, generates multimodel representation
% Original Author: Vitor Gazzaneo
% Last modified: September 10, 2019
% Modified by: Vitor Gazzaneo
% Modification: accepts generic dimensionalities 


function [M,d]= dT(U,Y,gx) 

m=size(U,2); % Identify dimensionality
p=size(Y,2); % Identify output dimensionaly
[Umin,Umax]=bounds(U); % Obtain input ranges

% Y=Y./[25 .45 6];


for j=1:1:m
    totU(:,j)=[linspace(Umin(j),Umax(j),gx)]; 
end % Evenly divide points in each dim.

h(1,:)=totU(2,:)-totU(1,:); % Obtain step in each dim.

Ind=[1:2]'; % Increment at each bottom left corner (min)
            % to compose a //polytope 
for i=1:1:m-1
    Add=[]; Stk=[];
    for j=1:1:2
        Add=[Add;ones(2^i,1).*j];
        Stk=[Stk; Ind];
    end
    Ind=[Add Stk];
end
Ind=Ind-1;

Ind_c=[1:gx-1]'; %Indices of bottom left corners
for i=1:1:m-1
    Add=[]; Stk=[];
    for j=1:1:gx-1
        Add=[Add;ones((gx-1)^i,1).*j];
        Stk=[Stk; Ind_c];
    end
    Ind_c=[Add Stk];
end

for I=1:1:(gx-1)^m % Each row of Corner
    for i=1:1:2^m  % Each row of Ind
        for j=1:1:m  % Each coordinate
            V_U(i,j)=totU(Ind_c(I,j),j)+Ind(i,j).*h(1,j);
        end
        [~,b]=ismember(round(V_U(i,:),8),round(U,8),'rows');
        V_Y(i,:)=Y(b,:);
    end
    if I==1&&m>3
        Inp=delaunayn(V_U,{'Qt', 'Qbb', 'Qc', 'Qz'}); %n-dim'l Triangulation 
        SpPll=size(Inp,1); %#Simplices in each //polytope
    elseif I==1&&m<=3
        Inp=delaunay(V_U);
        SpPll=size(Inp,1); %#Simplices in each //polytope
    end      
    
    for j=1:1:SpPll
        M(SpPll*(I-1)+j)=Polyhedron('V',[V_U(Inp(j,:),:) V_Y(Inp(j,:),:)]);
    end
    clear V_U V_Y;
end
 d=size(Inp,2); %#Vertices in each simplex

end

