% ITERATIVE ALGORITHM: FIND AN ELEMENT
% Purpose: find an opt. el't using MILP and multimodel rep'n
% Description: zooms in until solution converges
% Original Author: Vitor Gazzaneo
% Last modified: September 10, 2019
% Modified by: Vitor Gazzaneo
% Modification: accepts generic dimensionalities 

function [u_des,y_des] = dTOpt(U,Yd,mdl,tol,f_u,Ae_u,be_u,A_u,b_u)

m=size(U,2); p=size(Yd,2); % Identify dimensionalities

[Umin , Umax ]= bounds(U); 
[Ydmin, Ydmax]= bounds(Yd);  

A_sets=[-eye(m+p);eye(m+p)]; % Incorporates AIS and DOS as ineq. const's
b_sets=[-Umin';-Ydmin';Umax';Ydmax'];

Ai=[A_u;A_sets]; bi=[b_u;b_sets];

C_tot=Polyhedron('A',Ai,'b',bi,'Ae',Ae_u,'be',be_u); %(m+p) dim'l polytope -> Feasible region
% Here, from user entry, Ax=b, x =[u;y]. Also applies for f.

E_rel(:,1)=ones(m+p,1); % Initilization of loop
k=1;                    % Initilization of loop

% while sum(E_rel(:,end)>E_all)>0 % 
while any(E_rel(:,end)>tol)&&k<=50
    
clear f Aeq beq lb ub A b intcon t n ti t0 t1

gx=3;   
U=Evenly([Umin;Umax],gx);
Y=zeros(size(U,1),p);
for i=1:1:size(U,1) 
    i;
    Y(i,:)=mdl(U(i,:));
end
[M,d]=dT(U,Y,gx); %m+p dim'l polytopes representing the model
% plot(M(:).projection(1:m));     %AIS
% plot(M(:).projection(m+1:m+p)); %AOS

%% TRIAL TO CONSIDERS RELEVANT POLYTOPES
n=1;
for i=1:1:size(M,2)
        Test=PolyUnion('Set',[C_tot M(i)]);   
        if Test.isOverlapping==1
            ti(n,1)=i; n=n+1;
        end   
end
K=size(ti,1);

%% CREATING MILP PROBLEM
 clear f Aeq beq lb ub A b intcon

% EQUALITY CONSTRAINTS - AEQ
MD=zeros(m+p,K*d); % Model Data
SB=zeros(K  ,K*d); % Sum of Binary Var's

for j=1:1:K
    MD(:,(j-1)*d+1:j*d)=M(ti(j)).V';
    SB(j,(j-1)*d+1:j*d)=-ones(1,d);
end

if isempty(Ae_u) % Incorporates user entered Aeq 
    Ae_a=[];
else
    Ae_a=zeros(size(Ae_u,1),m+p+K+K*d);
    Ae_a(size(Ae_u,1),size(Ae_u,2))=Ae_u;
end

beq=[zeros(m+p,1); 1; zeros(K,1); be_u];

Aeq=[   -eye(m+p) zeros(m+p,K) MD
     zeros(1,m+p)    ones(1,K) zeros(1,K*d)
     zeros(K,m+p)       eye(K) SB  
     Ae_a                                   ];                                                                      

% INEQUALITY CONSTRAINTS - A
A=[Ai zeros(size(Ai,1),K+K*d)];
b=bi;

% OBJECTIVE FUNCTION - f
f=[f_u zeros(1,K+K*d)]; 

% BOUND CONSTRAINTS
lb=[-Inf(1,m+p)  zeros(1,K+K*d)];
ub=[+Inf(1,m+p)   ones(1,K+K*d)];

% BINARY VAR'S
intcon=(m+p+1:1:m+p+K)';

options = optimoptions('intlinprog','TolInteger',1e-3,'TolCon',1e-3);
[x]=intlinprog(f,intcon,A,b,Aeq,beq,lb,ub,options);
u(1:m,k)=x(1:m,1);
y(1:p,k)=x(m+1:m+p,1);

% CRITERIA
if k>1 
    E_rel(  1:m  ,k)=abs((u(1:m,k)-u(1:m,k-1))./(u(1:m,k)+u(1:m,k-1)));
    E_rel(m+1:m+p,k)=abs((y(1:p,k)-y(1:p,k-1))./(y(1:p,k)+y(1:p,k-1)))
end

% FIND THE SELECTED TETRAHEDRAL AND SEE ITS LIMITS FOR NEW U
[~,n]=ismember(1,round(x(m+p+1:m+p+K,1),2));
tsol=ti(n); % selected polytope

% Umin(1,1:m)=min(M(tsol).V(:,1:m)',[],2);  % Lower bound for new iteration
% Umax(1,1:m)=max(M(tsol).V(:,1:m)',[],2);  % Upper bound for new iteration 
[Umin(1,1:m),Umax(1,1:m)]=bounds(M(tsol).V(:,1:m))


k=k+1
end

u_des=u(1:m,end)';
y_des=y(1:p,end)';
end

